import QtQuick 2.12
import QtQuick.Controls 2.12
import QtQuick.Layouts 1.12
import QtQuick.Window 2.12
import QtCharts 2.3

import Generator 1.0
import Plotter 1.0

Window {
    height: 300
    width: 600

    title: qsTr("Fast Charting")

    visible: true

    Generator {
        id: generator
    }

    Plotter {
        id: plotter

        points: generator.points

        onPointsChanged: {
            updateSeries(lineSeries)
        }
    }

    ColumnLayout {
        anchors.fill: parent

        Slider {
            id: countSlider

            Layout.fillWidth: true

            snapMode: Slider.SnapAlways

            from: 1e3
            stepSize: from
            to: 1e7

            value: to

            onValueChanged: {
                if (generator.count !== value) {
                    generator.count = value
                }
            }
        }

        Button {
            id: generteButton

            Layout.alignment: Qt.AlignHCenter
            Layout.fillWidth: true

            text: "Generate and plot " + countSlider.value.toFixed(0) + " points"

            onClicked: {
                lineSeries.startTime = new Date().getTime()
                generator.generate()
            }
        }

        ChartView {
            id: chartView

            Layout.fillHeight: true
            Layout.fillWidth: true

            antialiasing: true

            axes: [ValueAxis, ValueAxis]

            backgroundRoundness: 0

            margins {
                bottom: 0
                left: 0
                right: 0
                top: 0
            }

            LineSeries {
                id: lineSeries

                property var startTime
                property var stopTime

                useOpenGL: true

                onPointsReplaced: {
                    stopTime = new Date().getTime()
                    name = "Qt " + qtversion + " generated " + generator.count + " points in " + generator.genrationTime +
                            " ms and plotted in " + (stopTime - startTime - generator.genrationTime) + " ms"
                }
            }

            Component.onCompleted: {
                axes[0].min = 0
                axes[0].max = Math.PI * 10
                axes[0].labelFormat = "%.2f"
                axes[1].min = -1.05
                axes[1].max = 1.05
                axes[1].labelFormat = "%.2f"
            }
        }
    }
}
